<?php

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Component\Display;
use EventPortal\Domain\Component\Dropdown\Select;
use EventPortal\Domain\Component\Input\Text;
use EventPortal\Domain\Component\Option;
use EventPortal\Domain\Component\Section;
use EventPortal\Domain\Delegate\Handlers\CreateDelegate;
use EventPortal\Domain\Delegate\Handlers\UpdateDelegate;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Handlers\CreateModel;
use EventPortal\Domain\Handlers\UpdateModel;
use EventPortal\Domain\User\Models\User;
use EventPortal\Enums\InertiaMethod;
use Illuminate\Validation\Rule;

Form::make('delegate.create')
    ->addHandler(new CreateModel)
    ->addHandler(new CreateDelegate)
    ->model(new User)
    ->addFields(function () {
        return [
            Section::make()->fields([
                Text::make('First name')
                    ->placeholder('Susan')
                    ->rules(['required']),
                Text::make('Last name')
                    ->placeholder('Gooseberry')
                    ->rules(['required']),
                Select::make('Delegate Type')
                    ->name('type')
                    ->placeholder('Please select delegate type')
                    ->options([
                        Option::make('Sales HCP'),
                        Option::make('Other'),
                    ]),
                Text::make('Email')
                    ->placeholder('susan.gooseberry@example.com')
                    ->rules(['required', 'email', 'unique:users,email']),
                Text::make('PA Email')
                    ->name('pa_email')
                    ->placeholder('personal-assistant@example.com')
                    ->rules(['nullable', 'email']),
            ]),
        ];
    })
    ->addActions(function () {
        return [
            Button::make('Cancel')
                ->variant('secondary')
                ->method(InertiaMethod::VISIT())
                ->route('admin.delegate.index'),
        ];
    })
    ->addPrimaryAction(function () {
        return Button::make('Add Delegate')
            ->method(InertiaMethod::POST())
            ->route('admin.delegate.store');
    });

Form::make('delegate.edit')
    ->addHandler(new UpdateModel)
    ->addHandler(new UpdateDelegate)
    ->addFields(function (Delegate $delegate) {
        return [
            Section::make()->fields([
                Text::make('First name')
                    ->placeholder('Susan')
                    ->scopeTo('user')
                    ->rules(['required']),
                Text::make('Last name')
                    ->placeholder('Gooseberry')
                    ->scopeTo('user')
                    ->rules(['required']),
                Select::make('Delegate Type')
                    ->name('type')
                    ->placeholder('Please select delegate type')
                    ->options([
                        Option::make('Sales HCP'),
                        Option::make('Other'),
                    ]),
                Text::make('Email')
                    ->placeholder('susan.gooseberry@example.com')
                    ->scopeTo('user')
                    ->rules([
                        'required',
                        'email',
                        Rule::unique('users', 'email')->ignore($delegate->user->id),
                    ]),
                Text::make('PA Email')
                    ->name('pa_email')
                    ->placeholder('personal-assistant@example.com')
                    ->rules(['nullable', 'email']),
            ]),
        ];
    })
    ->addActions(function () {
        return [
            Button::make('Cancel')
                ->variant('secondary')
                ->method(InertiaMethod::VISIT())
                ->route('admin.delegate.index'),
        ];
    })
    ->addPrimaryAction(function (Delegate $delegate) {
        return Button::make('Update Delegate')
            ->method(InertiaMethod::POST())
            ->route('admin.delegate.update', ['delegate' => $delegate]);
    });

Form::make('delegate.show')
    ->addFields(function () {
        return [
            Section::make()->fields([
                Display::make('First name')->scopeTo('user'),
                Display::make('Last name')->scopeTo('user'),
                Display::make('Email')->scopeTo('user'),
                Display::make('PA Email')->name('pa_email'),
            ]),
        ];
    })
    ->addActions(function (Delegate $delegate) {
        return [
            Button::make('Cancel')
                ->variant('secondary')
                ->method(InertiaMethod::VISIT())
                ->route('admin.delegate.index'),
            Button::make('Edit Delegate')
                ->method(InertiaMethod::VISIT())
                ->route('admin.delegate.edit', ['delegate' => $delegate]),
        ];
    });
