<?php

namespace EventPortal\Domain\Agenda\Rules;

use EventPortal\Domain\Agenda\Models\Pathway;
use EventPortal\Domain\Agenda\Models\Session;
use Illuminate\Contracts\Validation\Rule;

class NoOverlap implements Rule
{
    /** @var \EventPortal\Domain\Agenda\Models\Pathway */
    private $pathway;

    /** @var string */
    private $attribute;

    public function __construct(Pathway $pathway)
    {
        $this->pathway = $pathway;
    }

    /**
     * @param string $attribute
     * @param mixed $value
     * @return bool
     */
    // phpcs:disable SlevomatCodingStandard.TypeHints.ReturnTypeHint
    // phpcs:disable SlevomatCodingStandard.TypeHints.ParameterTypeHint
    public function passes($attribute, $value)
    {
        // phpcs:enable
        $this->attribute = $attribute;

        return Session::query()
            ->forPathway($this->pathway)
            ->partiallyOverlaps($value)
            ->exists() === false;
    }

    /**
     * @return array|string
     */
    public function message()
    {
        return "The {$this->getName()} cannot overlap another session.";
    }

    private function getName(): string
    {
        if ($this->attribute === 'starting_at') {
            return 'start time';
        }

        return 'end time';
    }
}
