<?php

namespace EventPortal\Http\Controllers\Admin\EventSpeaker;

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Speaker\Models\Speaker;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Domain\Util\AdminRender;
use EventPortal\Domain\Util\Breadcrumb;
use EventPortal\Domain\Util\Sidebar\Facades\Sidebar;
use EventPortal\Enums\InertiaMethod;
use EventPortal\Http\Controllers\Controller;
use Illuminate\Database\Query\JoinClause;
use Inertia\Response;

class EventSpeakerController extends Controller
{
    public function index(Event $event): Response
    {
        $query = Speaker::query()
            ->join('event_speaker', static function (JoinClause $query) {
                return $query->on('speakers.id', '=', 'event_speaker.speaker_id');
            })
            ->joinRelationship('user')
            ->where('event_speaker.event_id', '=', $event->id);

        $table = Table::make('event-speaker.index')
            ->context($event)
            ->query($query)
            ->request(request())
            ->get();

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add($event->name, route('admin.event.show', $event))
            ->add('Speakers')
            ->get();

        $sidebar = Sidebar::make('event.show')
            ->user(auth()->user())
            ->model($event)
            ->render();

        $quickButton = Button::make('Add speaker to event')
            ->method(InertiaMethod::VISIT())
            ->icon('plus')
            ->route('admin.event.speaker.create', $event);

        return AdminRender::component('Admin/EventSpeaker/Index')
            ->key('event.speaker.index')
            ->withTable($table)
            ->withBreadcrumbs($breadcrumbs)
            ->withSideBar($sidebar)
            ->attributes(['event' => $event])
            ->withQuickButton($quickButton, has_permission('speaker.store'))
            ->get();
    }

    public function show(Event $event, Speaker $speaker): Response
    {
        $speakerData = Form::make('event-speaker.show')
            ->model($speaker)
            ->context($event)
            ->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add($event->name, route('admin.event.show', $event))
            ->add('Speakers', route('admin.event.speaker.index', $event))
            ->add('View Speaker')
            ->get();

        $sidebar = Sidebar::make('event.show')
            ->user(auth()->user())
            ->model($event)
            ->render();

        return AdminRender::component('Admin/EventSpeaker/Show')
            ->key('event.speaker.show')
            ->withData($speakerData)
            ->withBreadcrumbs($breadcrumbs)
            ->withTitle($speaker->name)
            ->withSideBar($sidebar)
            ->get();
    }
}
